@@ -4,14 +4,14 @@
 from __future__ import annotations
 
 __author__ = "ChenyangGao <https://chenyanggao.github.io>"
-__version__ = (0, 0, 5)
+__version__ = (0, 0, 6)
 __all__ = [
     "SizedDict", "LIFODict", "FIFODict", "RRDict", "LRUDict", "MRUDict", 
     "TTLDict", "LFUDict", "PriorityDict", "ExpireDict", "TLRUDict", 
     "FastFIFODict", "FastLRUDict", 
 ]
 
-from collections import deque
+from collections import deque, UserDict
 from collections.abc import Callable, Hashable, Iterable, Iterator
 from copy import copy
 from dataclasses import dataclass, field
@@ -21,7 +21,7 @@ from inspect import signature, _empty
 from itertools import count
 from math import inf, isinf, isnan
 from operator import itemgetter
-from random import randrange
+from random import choice
 from time import time
 from types import MappingProxyType
 from typing import cast, overload, Any, Literal, Self
@@ -142,7 +142,9 @@ class SizedDict[K: Hashable, V](dict[K, V]):
         if self.auto_clean:
             self.clean()
         cls = type(self)
-        return f"<{cls.__module__}.{cls.__qualname__} object at {hex(id(self))} with {super().__repr__()}>"
+        maxsize = self.maxsize
+        auto_clean = self.auto_clean
+        return f"<{cls.__module__}.{cls.__qualname__}({maxsize=!r}, {auto_clean=!r}) object at {hex(id(self))} with {super().__repr__()}>"
 
     def __contains__(self, key, /) -> bool:
         if self.auto_clean:
@@ -154,7 +156,7 @@ class SizedDict[K: Hashable, V](dict[K, V]):
             return False
 
     def __setitem__(self, key: K, value: V, /):
-        if self.auto_clean and super().__contains__(key):
+        if self.auto_clean and not super().__contains__(key):
             self.clean(1)
         super().__setitem__(key, value)
 
@@ -343,24 +345,58 @@ class FIFODict[K: Hashable, V](LIFODict[K, V]):
 
 
 class RRDict[K: Hashable, V](SizedDict[K, V]):
-    __slots__ = ("maxsize", "auto_clean")
+    __slots__ = ("maxsize", "auto_clean", "_keys", "_key_to_idx")
+
+    def __init__(
+        self, 
+        /, 
+        maxsize: int = 0, 
+        auto_clean: bool = True, 
+    ):
+        super().__init__(maxsize, auto_clean)
+        self._keys: list[K] = []
+        self._key_to_idx: dict[K, int] = {}
+
+    def __copy__(self, /) -> Self:
+        inst = super().copy()
+        inst._keys = copy(self._keys)
+        inst._key_to_idx = copy(self._key_to_idx)
+        return inst
+
+    def __delitem__(self, key: K, /):
+        super().__delitem__(key)
+        i = self._key_to_idx.pop(key)
+        last_key = self._keys.pop()
+        if key is not last_key:
+            self._keys[i] = last_key
+            self._key_to_idx[last_key] = i
+
+    def __setitem__(self, key: K, value: V, /):
+        super().__setitem__(key, value)
+        if key not in self._key_to_idx:
+            keys = self._keys
+            self._key_to_idx[key] = len(keys)
+            keys.append(key)
 
     def popitem(self, /) -> tuple[K, V]:
+        key = self.key
+        return key, self.pop(key)
+
+    @property
+    def key(self, /) -> K:
         try:
-            while True:
-                try:
-                    idx = randrange(len(self))
-                    for i, key in enumerate(self):
-                        if i == idx:
-                            break
-                    return key, self.pop(key)
-                except CleanedKeyError as e:
-                    return e.key, e.value
-                except (KeyError, RuntimeError):
-                    pass
-        except StopIteration:
-            pass
-        raise KeyError(f"{self!r} is empty")
+            return choice(self._keys)
+        except IndexError as e:
+            raise KeyError(f"{self!r} is empty") from e
+
+    @property
+    def value(self, /) -> V:
+        return self[self.key]
+
+    @property
+    def item(self, /) -> tuple[K, V]:
+        key = self.key
+        return key, self[key]
 
 
 class LRUDict[K: Hashable, V](FIFODict[K, V]):
@@ -536,11 +572,31 @@ class TTLDict[K, V](SizedDict[K, V]):
         return items
 
 
-class Counter[K](dict[K, int]):
+class Counter[K: Hashable](UserDict[K, int]):
+
+    def __init__(self, /):
+        self.data = {}
+        self._heap: list[KeyPriority[int, K]] = []
+        self._key_to_entry: dict[K, KeyPriority[int, K]] = {}
 
     def __missing__(self, _: K, /) -> Literal[0]:
         return 0
 
+    def __delitem__(self, key: K, /):
+        super().__delitem__(key)
+        if entry := self._key_to_entry.pop(key, None):
+            entry.key = undefined
+
+    def __setitem__(self, key: K, value: int, /):
+        if value <= 0:
+            del self[key]
+        else:
+            if entry := self._key_to_entry.pop(key, None):
+                entry.key = undefined
+            super().__setitem__(key, value)
+            entry = self._key_to_entry[key] = KeyPriority(value, key=key)
+            heappush(self._heap, entry)
+
     def max(self, /) -> tuple[K, int]:
         try:
             return max(self.items(), key=itemgetter(1))
@@ -548,10 +604,17 @@ class Counter[K](dict[K, int]):
             raise KeyError(f"{self!r} is empty") from e
 
     def min(self, /) -> tuple[K, int]:
+        heap = self._heap
         try:
-            return min(self.items(), key=itemgetter(1))
-        except ValueError as e:
-            raise KeyError(f"{self!r} is empty") from e
+            while True:
+                key = heap[0].key
+                if key is not undefined:
+                    key = cast(K, key)
+                    return key, self[key]
+                heappop(heap)
+        except IndexError:
+            pass
+        raise KeyError(f"{self!r} is empty")
 
     def most_common(
         self, 
